//+------------------------------------------------------------------+
//|                        Smoothed Rsi Inverse Fisher Transform.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
#property strict

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  clrSilver
#property indicator_color2  clrLimeGreen
#property indicator_color3  clrLimeGreen
#property indicator_color4  clrOrange
#property indicator_color5  clrOrange
#property indicator_color6  clrGold
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property indicator_width4  3
#property indicator_width5  3
#property indicator_minimum -5
#property indicator_maximum 105

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased   // Heiken ashi trend biased price
};

extern ENUM_TIMEFRAMES TimeFrame = PERIOD_CURRENT;  // Time frame
extern int      Length           = 14;              // Rsx period
extern int      MomLength        = 4;               // Momentum period
extern bool     UseFilteredPrice = false;           // Use rwma smoothed price?
extern int      EmaPeriod        = 4;               // Ema period
extern int      SignalPeriod     = 5;               // Signal period
extern int      RwmaLength       = 2;               // Rwma period (if used for smoothing the price)
extern int      Depth            = 10;              // Rwma depth of calculation 
extern enPrices Price            = pr_close;        // Price
extern double   LevelUp          = 88;              // Upper level
extern double   LevelDown        = 12;              // Lower level
extern bool     alertsOn        = false;            // Turn alerts on?
extern bool     alertsOnCurrent = false;            // Alert on current (still opened) bar?
extern bool     alertsMessage   = true;             // Alerts should display messages?
extern bool     alertsSound     = false;            // Alerts should play a sound
extern bool     alertsEmail     = false;            // Alerts should send an email?
extern bool     Interpolate      = true;            // Interpolate in multi time frame mode?


//
//
//
//
//

#define MAX_depth 50
double  inv[];
double  invUa[];
double  invUb[];
double  invDa[];
double  invDb[];
double  trend[];
double  trends[];
double  signal[];
double  pBuffer[][MAX_depth];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
      IndicatorBuffers(8);
      SetIndexBuffer(0,inv);
      SetIndexBuffer(1,invUa);
      SetIndexBuffer(2,invUb);
      SetIndexBuffer(3,invDa);
      SetIndexBuffer(4,invDb);
      SetIndexBuffer(5,signal);
      SetIndexBuffer(6,trend);
      SetIndexBuffer(7,trends);
         SetLevelValue(0,LevelUp);
         SetLevelValue(1,LevelDown);
      
      //
      //
      //
      //
      //
      
      RwmaLength = MathMax(RwmaLength,2);
      Depth      = MathMax(MathMin(Depth,MAX_depth),2);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);  
            
      //
      //
      //
      //
      //
        
   IndicatorShortName(timeFrameToString(TimeFrame)+ " Rwma Smoothed Rsx Inverse Fisher Transform ("+(string)Length+","+(string)EmaPeriod+","+(string)RwmaLength+","+(string)Depth+")");
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrkBuffer[][13];
double work[][2];
#define _ema1 0
#define _ema2 1

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,l,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { inv[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   if (TimeFrame == Period())
   {
      if (ArrayRange(pBuffer,0) != Bars) { ArrayResize(pBuffer,Bars); ArrayResize(work,Bars); ArrayResize(wrkBuffer,Bars); }
      if (trend[limit]== 1) CleanPoint(limit,invUa,invUb);
      if (trend[limit]==-1) CleanPoint(limit,invDa,invDb);
      double weight = (RwmaLength+1.0)*RwmaLength*0.5;
      
      //
      //
      //
      //
      //
      
      double alpha  = 2.0 / (1.0+EmaPeriod);
      double alphas = 2.0 / (1.0+SignalPeriod);
      double Kg     = (3.0)/(2.0+Length); 
      double Hg     = 1.0-Kg;
      for(i=limit, r=Bars-i-1; i>=0; i--, r++)
      {
         double price = getPrice(Price,Open,Close,High,Low,i);
         if (UseFilteredPrice)
         {
            double coeff  = 5;
            double sum    = 0;
            double sumc   = 0;
            for (int s=0; s<Depth; s++)
            {
               pBuffer[r][s] = price;
               if (r>RwmaLength)
               {
                  for (l=0, price=0; l<RwmaLength; l++) price += (RwmaLength-l)*pBuffer[r-l][s];
                                                        price /= weight;
               }
               sum  += coeff*price;
               sumc += coeff;
               coeff = MathMax(coeff-1,1);
            }
            wrkBuffer[r][12] = sum/sumc;
         }
         else  wrkBuffer[r][12] = price;             
         if (i==(Bars-1)) { for (int c=0; c<12; c++) wrkBuffer[r][c] = 0; continue; }  

      //
      //
      //
      //
      //
      
      double mom = wrkBuffer[r][12]-wrkBuffer[MathMax(r-MomLength,0)][12];
      double moa = MathAbs(mom);
      for (int k=0; k<3; k++)
      {
         int kk = k*2;
            wrkBuffer[r][kk+0] = Kg*mom                + Hg*wrkBuffer[r-1][kk+0];
            wrkBuffer[r][kk+1] = Kg*wrkBuffer[r][kk+0] + Hg*wrkBuffer[r-1][kk+1]; mom = 1.5*wrkBuffer[r][kk+0] - 0.5 * wrkBuffer[r][kk+1];
            wrkBuffer[r][kk+6] = Kg*moa                + Hg*wrkBuffer[r-1][kk+6];
            wrkBuffer[r][kk+7] = Kg*wrkBuffer[r][kk+6] + Hg*wrkBuffer[r-1][kk+7]; moa = 1.5*wrkBuffer[r][kk+6] - 0.5 * wrkBuffer[r][kk+7];
      }
      double rsx = 0.1 * ((50.0)-50);
      if (moa != 0)
           rsx = 0.1 * (MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)-50); 
      work[r][_ema1] = work[r-1][_ema1]+alpha*(rsx           -work[r-1][_ema1]);
      work[r][_ema2] = work[r-1][_ema2]+alpha*(work[r][_ema1]-work[r-1][_ema1]);
      double zlema          = 2.0*work[r][_ema1]-work[r][_ema2];
             inv[i]         = 50.0*(1+(MathExp(2*zlema)-1)/(MathExp(2*zlema)+1));
             signal[i]      = signal[i+1]+alphas*(inv[i]-signal[i+1]);
             invUa[i]       = EMPTY_VALUE;
             invUb[i]       = EMPTY_VALUE;
             invDa[i]       = EMPTY_VALUE;
             invDb[i]       = EMPTY_VALUE;
             trend[i]       = 0;
             trends[i]      = trends[i+1];
 
             if (inv[i] > LevelUp)   trend[i] = 1;
             if (inv[i] < LevelDown) trend[i] =-1;
             if (inv[i] > signal[i]) trends[i]= 1;
             if (inv[i] < signal[i]) trends[i]=-1;
             if (trend[i]== 1) PlotPoint(i,invUa,invUb,inv);
             if (trend[i]==-1) PlotPoint(i,invDa,invDb,inv);
      }      
      manageAlerts();      
      return(0);
   }

   //
   //
   //
   //
   //
   
   limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (trend[limit]== 1) CleanPoint(limit,invUa,invUb);
   if (trend[limit]==-1) CleanPoint(limit,invDa,invDb);
   for (i=limit;i>=0; i--)
   {
      int y     = iBarShift(NULL,TimeFrame,Time[i]);
      inv[i]    = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
      signal[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,5,y);
      invUa[i]  = EMPTY_VALUE;
      invUb[i]  = EMPTY_VALUE;
      invDa[i]  = EMPTY_VALUE;
      invDb[i]  = EMPTY_VALUE;
      trend[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,6,y);
      trends[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,7,y);
      
      //
      //
      //
      //
      //

      if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                  
      //
      //
      //
      //
      //
                  
      int n,j; datetime time = iTime(NULL,TimeFrame,y);
      for(n = 1; (i+n)<Bars && Time[i+n] >= time; n++) continue;	
      for(j = 1; j<n && (i+n)<Bars && (i+j)<Bars; j++)
      {
         inv[i+j]    = inv[i]    + (inv[i+n]   -inv[i]   )*j/n;
         signal[i+j] = signal[i] + (signal[i+n]-signal[i])*j/n;
      }                           
   }
   for (i=limit;i>=0;i--)
   {
      if (trend[i]== 1) PlotPoint(i,invUa,invUb,inv);
      if (trend[i]==-1) PlotPoint(i,invDa,invDb,inv);
   }
   return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

double workHa[][4];
double getPrice(int price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (price>=pr_haclose)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars);
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (price)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
            
      static datetime time1 = 0;
      static string   mess1 = "";
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(time1,mess1,whichBar,"crossed level up");
         if (trend[whichBar] == -1) doAlert(time1,mess1,whichBar,"crossed level down");
      }
      static datetime time2 = 0;
      static string   mess2 = "";
      if (trends[whichBar] != trends[whichBar+1])
      {
         if (trends[whichBar] ==  1) doAlert(time2,mess2,whichBar,"crossed signal line up");
         if (trends[whichBar] == -1) doAlert(time2,mess2,whichBar,"crossed signal line down");
      }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message = _Symbol+" "+timeFrameToString(_Period)+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Rwma Rsi Inv Fisher "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Rwma Rsi Inv Fisher "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

  
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}